@@ -56,12 +56,6 @@ struct mag3110_data {
 	int sleep_val;
 	struct regulator *vdd_reg;
 	struct regulator *vddio_reg;
-	/* Ensure natural alignment of timestamp */
-	struct {
-		__be16 channels[3];
-		u8 temperature;
-		s64 ts __aligned(8);
-	} scan;
 };
 
 static int mag3110_request(struct mag3110_data *data)
@@ -393,9 +387,10 @@ static irqreturn_t mag3110_trigger_handler(int irq, void *p)
 	struct iio_poll_func *pf = p;
 	struct iio_dev *indio_dev = pf->indio_dev;
 	struct mag3110_data *data = iio_priv(indio_dev);
+	u8 buffer[16]; /* 3 16-bit channels + 1 byte temp + padding + ts */
 	int ret;
 
-	ret = mag3110_read(data, data->scan.channels);
+	ret = mag3110_read(data, (__be16 *) buffer);
 	if (ret < 0)
 		goto done;
 
@@ -404,10 +399,10 @@ static irqreturn_t mag3110_trigger_handler(int irq, void *p)
 			MAG3110_DIE_TEMP);
 		if (ret < 0)
 			goto done;
-		data->scan.temperature = ret;
+		buffer[6] = ret;
 	}
 
-	iio_push_to_buffers_with_timestamp(indio_dev, &data->scan,
+	iio_push_to_buffers_with_timestamp(indio_dev, buffer,
 		iio_get_time_ns(indio_dev));
 
 done:
