@@ -42,7 +42,7 @@ struct iavf_stats {
  */
 #define IAVF_STAT(_type, _name, _stat) { \
 	.stat_string = _name, \
-	.sizeof_stat = FIELD_SIZEOF(_type, _stat), \
+	.sizeof_stat = sizeof_field(_type, _stat), \
 	.stat_offset = offsetof(_type, _stat) \
 }
 
@@ -278,18 +278,7 @@ static int iavf_get_link_ksettings(struct net_device *netdev,
 	ethtool_link_ksettings_zero_link_mode(cmd, supported);
 	cmd->base.autoneg = AUTONEG_DISABLE;
 	cmd->base.port = PORT_NONE;
-	cmd->base.duplex = DUPLEX_FULL;
-
-	if (ADV_LINK_SUPPORT(adapter)) {
-		if (adapter->link_speed_mbps &&
-		    adapter->link_speed_mbps < U32_MAX)
-			cmd->base.speed = adapter->link_speed_mbps;
-		else
-			cmd->base.speed = SPEED_UNKNOWN;
-
-		return 0;
-	}
-
+	/* Set speed and duplex */
 	switch (adapter->link_speed) {
 	case IAVF_LINK_SPEED_40GB:
 		cmd->base.speed = SPEED_40000;
@@ -317,6 +306,7 @@ static int iavf_get_link_ksettings(struct net_device *netdev,
 	default:
 		break;
 	}
+	cmd->base.duplex = DUPLEX_FULL;
 
 	return 0;
 }
@@ -962,13 +952,14 @@ static int iavf_get_rxfh(struct net_device *netdev, u32 *indir, u8 *key,
 
 	if (hfunc)
 		*hfunc = ETH_RSS_HASH_TOP;
-	if (key)
-		memcpy(key, adapter->rss_key, adapter->rss_key_size);
+	if (!indir)
+		return 0;
 
-	if (indir)
-		/* Each 32 bits pointed by 'indir' is stored with a lut entry */
-		for (i = 0; i < adapter->rss_lut_size; i++)
-			indir[i] = (u32)adapter->rss_lut[i];
+	memcpy(key, adapter->rss_key, adapter->rss_key_size);
+
+	/* Each 32 bits pointed by 'indir' is stored with a lut entry */
+	for (i = 0; i < adapter->rss_lut_size; i++)
+		indir[i] = (u32)adapter->rss_lut[i];
 
 	return 0;
 }
